package extension

import (
	"git.perx.ru/perxis/perxis-go/pkg/references"
	pb "git.perx.ru/perxis/perxis-go/proto/extensions"
	"github.com/mitchellh/mapstructure"
)

const ActionsCollectionID = "space_actions"

type (
	ActionKind = pb.Action_Kind
	//ActionRequest  = pb.ActionRequest
	//ActionResponse = pb.ActionResponse
)

const (
	ResponseDone               = pb.ActionResponse_DONE
	ResponseError              = pb.ActionResponse_ERROR
	ResponsePending            = pb.ActionResponse_PENDING
	ResponseInProgress         = pb.ActionResponse_IN_PROGRESS
	ResponseParametersRequired = pb.ActionResponse_PARAMETERS_REQUIRED
)

type Action struct {
	Extension        string                  `mapstructure:"extension,omitempty"`   // Расширение
	Action           string                  `mapstructure:"action,omitempty"`      // Идентификатор действия
	Name             string                  `mapstructure:"name,omitempty"`        // Название действия для отображения в интерфейсе (пункт меню, кнопка).
	Description      string                  `mapstructure:"description,omitempty"` // Описание действия для отображения в интерфейсе
	Icon             string                  `mapstructure:"icon,omitempty"`        // Название иконки для отображения действия в интерфейсе
	Image            *references.Reference   `mapstructure:"image,omitempty"`       // Изображение для отображения в действия в интерфейсе
	Groups           []string                `mapstructure:"groups,omitempty"`      // Группы отображения действия в интерфейсе
	Kind             ActionKind              `mapstructure:"kind,omitempty"`        // Указывает на что направлено действие
	Classes          []string                `mapstructure:"classes,omitempty"`     // Классы данных к которым применимо действие (название коллекций или специальных групп в рамках которых данное действие применимо)
	Refs             []*references.Reference `mapstructure:"refs,omitempty"`        // Ссылки на записи используемые для выполнения действия (назначение ссылок зависит от действия и расширения)
	ParamsCollection string                  `mapstructure:"params_collection,omitempty"`
	Request          *ActionRequest          `mapstructure:"request,omitempty"`           // Параметры запроса (используется в случае `ActionResponse.next`)
	NavigationAction bool                    `mapstructure:"navigation_action,omitempty"` // Флаг указывающий что действие переносить пользователя в другую часть интерфейса, а не отправляет запрос на сервер
	NavigationRoute  string                  `mapstructure:"navigation_route,omitempty"`
}

func ActionToMap(action *Action) map[string]interface{} {
	res := make(map[string]interface{})
	mapstructure.Decode(action, &res)
	res["kind"] = int64(action.Kind.Number())
	return res
}

func ActionFromMap(d map[string]interface{}) (*Action, error) {
	var action Action
	err := mapstructure.Decode(d, &action)
	return &action, err
}

func ActionFromPB(a *pb.Action) *Action {
	return &Action{
		Extension:        a.Extension,
		Action:           a.Action,
		Name:             a.Name,
		Description:      a.Description,
		Icon:             a.Icon,
		Image:            references.ReferenceFromPB(a.Image),
		Groups:           a.Groups,
		Kind:             a.Kind,
		Classes:          a.Classes,
		Refs:             references.ReferenceListFromPB(a.Refs),
		ParamsCollection: a.ParamsCollection,
		Request:          a.Request,
		NavigationAction: a.NavigationAction,
		NavigationRoute:  a.NavigationRoute,
	}
}
